/**********************************************************************
** Copyright (C) 2000-2002 Trolltech AS.  All rights reserved.
**
** This file is part of the Qtopia Environment.
**
** This file may be distributed and/or modified under the terms of the
** GNU General Public License version 2 as published by the Free Software
** Foundation and appearing in the file LICENSE.GPL included in the
** packaging of this file.
**
** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
**
** See http://www.trolltech.com/gpl/ for GPL licensing information.
**
** Contact info@trolltech.com if any conditions of this licensing are
** not clear to you.
**
**********************************************************************/

#include "clock.h"

#include <qpe/global.h>
#include <qpe/qpeapplication.h>
#include <qpe/config.h>
#include <qpe/qcopenvelope_qws.h>

#include <qmainwindow.h>
#include <qlayout.h>
#include <qpushbutton.h>
#include <qdatetime.h>
#include <qtimer.h>
#include <qpopupmenu.h>
#include <stdlib.h>
#include <qtextcodec.h>
#include <qcombobox.h>
#include <qframe.h>
#include <qlayout.h>
#include <qspinbox.h>

LauncherClock::LauncherClock( QWidget *parent ) : QLabel( parent )
{
	// եɤ߹
	loadConfig();

    connect( qApp, SIGNAL( timeChanged() ), this, SLOT( updateTime( ) ) );
    connect( qApp, SIGNAL( clockChanged( bool ) ),
	     this, SLOT( slotClockChanged( bool ) ) );

	// ݥåץåץ˥塼
	QTextCodec* codec = QTextCodec::codecForName("eucJP");
    menu = new QPopupMenu(this);
	menu->insertItem(codec->toUnicode(""), this, SLOT(slotShowTimeSetting()));
	menu->insertItem(codec->toUnicode("ɽ"), this, SLOT(slotShowSetting()));

	refreshApplet();
    show();
}
void LauncherClock::refreshApplet()
{
	// ե
    setFont( QFont( mFontName, mFontSize, QFont::Bold ) );

	// 
	QPalette defaultPalette(palette());
	defaultPalette.setColor(QColorGroup::Foreground,QColor(mFontColor_R,mFontColor_G,mFontColor_B));
	setPalette(defaultPalette);

    Config config( "qpe" );
    config.setGroup( "Time" );
    ampmFormat = config.readBoolEntry( "AMPM", TRUE );
    timerId = 0;
    timerEvent( 0 );

}

void LauncherClock::loadConfig()
{

	// ͤν
	mFontName = "lcfont";
	mFontSize = 20;
	mFontColor_R = 0;
	mFontColor_G = 0;
	mFontColor_B = 0;
    mDayEnable = true;
    mYearView = true;
    mYearFull = true;
    mWeekView = true;
    mWeekJapan = true;
    mWordWrap = false;
    mSecView = false;
    mYearEto = false;

	// configեɤ߹
    Config config( "clockapplet" );

	// Fontط
    config.setGroup( "FontSetting" );
	mFontName = config.readEntry("FontName", mFontName);
	mFontSize = config.readNumEntry("FontSize", mFontSize);
	mFontColor_R = config.readNumEntry("FontColor_R", mFontColor_R);
	mFontColor_G = config.readNumEntry("FontColor_G", mFontColor_G);
	mFontColor_B = config.readNumEntry("FontColor_B", mFontColor_B);

	// ɽط
    config.setGroup( "ViewSetting" );
	mDayEnable = config.readBoolEntry("Enable", mDayEnable);
	mYearView = config.readBoolEntry("YearView", mYearView);
	mYearFull = config.readBoolEntry("YearFull", mYearFull);
	mWeekView = config.readBoolEntry("WeekView", mWeekView);
	mWeekJapan = config.readBoolEntry("WeekJapan", mWeekJapan);
	mWordWrap = config.readBoolEntry("WordWrap", mWordWrap);
	mSecView = config.readBoolEntry("SecView", mSecView);
	mYearEto = config.readBoolEntry("YearEto", mYearEto);
    
	return;
}

void LauncherClock::saveConfig()
{
	// configեؽ񤭽Ф
	Config config("clockapplet");

	// Fontط
    config.setGroup( "FontSetting" );
	config.writeEntry("FontName", mFontName);
	config.writeEntry("FontSize", mFontSize);
	config.writeEntry("FontColor_R", mFontColor_R);
	config.writeEntry("FontColor_G", mFontColor_G);
	config.writeEntry("FontColor_B", mFontColor_B);

	// ɽط
    config.setGroup( "ViewSetting" );
	config.writeEntry("Enable", mDayEnable);
	config.writeEntry("YearView", mYearView);
	config.writeEntry("YearFull", mYearFull);
	config.writeEntry("WeekView", mWeekView);
	config.writeEntry("WeekJapan", mWeekJapan);
	config.writeEntry("WordWrap", mWordWrap);
	config.writeEntry("SecView", mSecView);
	config.writeEntry("YearEto", mYearEto);

	return;

}

void LauncherClock::slotShowSetting()
{
	OptionDialog optDialog( this, "", true, 0 );
	optDialog.setFontName( mFontName );
	optDialog.setFontSize( mFontSize );
	optDialog.setFontColor_R( mFontColor_R );
	optDialog.setFontColor_G( mFontColor_G );
	optDialog.setFontColor_B( mFontColor_B );
	optDialog.setDayEnable( mDayEnable );
	optDialog.setYearView( mYearView );
	optDialog.setYearFull( mYearFull );
	optDialog.setWeekView( mWeekView );
	optDialog.setWeekJapan( mWeekJapan );
	optDialog.setWordWrap( mWordWrap );
	optDialog.setSecView( mSecView );
	optDialog.setYearEto( mYearEto );

	if ( optDialog.exec() == QDialog::Accepted )
	{
		mFontName = optDialog.getFontName();
		mFontSize = optDialog.getFontSize();
		mFontColor_R = optDialog.getFontColor_R();
		mFontColor_G = optDialog.getFontColor_G();
		mFontColor_B = optDialog.getFontColor_B();
		mDayEnable = optDialog.getDayEnable();
		mYearView = optDialog.getYearView();
		mYearFull = optDialog.getYearFull();
		mWeekView = optDialog.getWeekView();
		mWeekJapan = optDialog.getWeekJapan();
		mWordWrap = optDialog.getWordWrap();
		mSecView = optDialog.getSecView();
		mYearEto = optDialog.getYearEto();

		refreshApplet();
		saveConfig();
	}	
//	QCopEnvelope("QPE/TaskBar", "reloadApplets()");
 	return;
}

void LauncherClock::slotShowTimeSetting()
{
    Global::execute( "systemtime" );
	refreshApplet();
	return;
}


void LauncherClock::mouseReleaseEvent( QMouseEvent * )
{
  QPoint curPos = mapToGlobal( QPoint(0,0) );
  QSize sh = menu->sizeHint();
  menu->popup( curPos-QPoint((sh.width()-width())/2,sh.height()) );
	return;

}


void LauncherClock::timerEvent( QTimerEvent *e )
{
    int intervalSec = 1;
	if(mSecView)
		intervalSec = 60;

    if ( !e || e->timerId() == timerId ) {
	killTimer( timerId );
	changeTime();
	QTime t = QTime::currentTime();

	int ms = ((60 - t.second())*1000 - t.msec() ) / intervalSec;
	timerId = startTimer( ms );
    } else {
	QLabel::timerEvent( e );
    }
}

void LauncherClock::updateTime( void )
{
    changeTime();
}

void LauncherClock::changeTime( void )
{
    QTime tm = QDateTime::currentDateTime().time();
    QDate dt = QDate::currentDate();
    QString strYear = "";
    QString strDay = "";
    QString strTime = "";
    QString strWeek = "";
    QString strView = "";
    int intEto = 0;
    static const char* conWeek[] = { "", "", "", "", "", "", ""};
    static const char* conEto[] = { "", "", "", "", "", "", "", "", "ä", "̦", "", "" };

	QTextCodec* codec = QTextCodec::codecForName("eucJP");

    if( ampmFormat ) {
		int hour = tm.hour(); 
		if (hour == 0)
		    hour = 12;
		if (hour > 12)
		    hour -= 12;
		if(mSecView)
			strTime.sprintf( "%2d%c%02d%c%02d %s", hour, ':', tm.minute(),':', tm.second(), (tm.hour() >= 12) ? "PM" : "AM" );
		else
			strTime.sprintf( "%2d%c%02d %s", hour, ':', tm.minute(), (tm.hour() >= 12) ? "PM" : "AM" );
    }else{
		if(mSecView)
			strTime.sprintf( "%2d%c%02d%c%02d",tm.hour(), ':', tm.minute() ,':', tm.second());
		else
			strTime.sprintf( "%2d%c%02d",tm.hour(), ':', tm.minute() );
	}

	if(mDayEnable){
		if(mYearView){
			if(mYearFull)
				strYear.sprintf( "%4d",dt.year());
			else
				strYear.sprintf( "%02d",(dt.year()-2000));

			if(mYearEto){
				intEto = dt.year() % 12;
				strYear = strYear + "(" + codec->toUnicode(conEto[intEto]) + ")";
			}else
				strYear = strYear + "/";

		}
		strDay.sprintf( "%02d/%02d", dt.month(), dt.day());

		if(mWeekView){
			if(mWeekJapan)
				strWeek = "(" + codec->toUnicode(conWeek[dt.dayOfWeek()-1])  + ")";
			else
				strWeek = "(" + dt.dayName(dt.dayOfWeek())  + ")";
		}else
			strWeek = " ";

		if(mWordWrap)
			strTime = "\n" + strTime;
	}

	strView = strYear + strDay + strWeek + strTime;
    setText( strView );

}

void LauncherClock::slotClockChanged( bool pm )
{
    ampmFormat = pm;
    updateTime();
}




OptionDialog::OptionDialog( QWidget* parent, const char* name, bool modal, WFlags fl )
    : QDialog( parent, name, modal, fl )
{

	static const char* item1[] = { "ʤ", "" };
	static const char* item2[] = {  "07", "2007" };
	static const char* item3[] = { "(Mon)", "()" };

	QTextCodec	*codec;
	int	i;

	codec = QTextCodec::codecForName( "eucJP" );
	setCaption( codec->toUnicode( "ɽ" ) );

    OptionLayout = new QVBoxLayout( this ); 
    OptionLayout->setSpacing( 6 );
    OptionLayout->setMargin( 11 );

    FontSetting = new QGridLayout; 
    FontSetting->setSpacing( 3 );
    FontSetting->setMargin( 0 );

	QLabel* label1_1 = new QLabel( codec->toUnicode( "ե̾" ), this );
	FontSetting->addWidget( label1_1, 0, 0 );
	FontName = new QComboBox( false, this );
    FontSetting->addWidget( FontName, 0, 1 );

	QLabel* label1_2 = new QLabel( codec->toUnicode( "եȥ" ), this );
	FontSetting->addWidget( label1_2 , 1, 0 );
	FontSize = new QComboBox( false, this );
    FontSetting->addWidget( FontSize, 1, 1 );

	QLabel* label1_3 = new QLabel( codec->toUnicode( "եȿ" ), this );
	FontSetting->addWidget( label1_3 , 2, 0 );

    ColorLayout = new QHBoxLayout( this ); 
    ColorLayout->setSpacing( 6 );
    ColorLayout->setMargin( 11 );

	QLabel* label_R = new QLabel( codec->toUnicode( "R:" ), this );
	ColorLayout->addWidget( label_R );

    FontColor_R = new QSpinBox( this, "FontColor_R" );
    FontColor_R->setMaxValue(255);
    FontColor_R->setMinValue(0);
	ColorLayout->addWidget( FontColor_R );

	QLabel* label_G = new QLabel( codec->toUnicode( "G:" ), this );
	ColorLayout->addWidget( label_G );

    FontColor_G = new QSpinBox( this, "FontColor_G" );
    FontColor_G->setMaxValue(255);
    FontColor_G->setMinValue(0);
	ColorLayout->addWidget( FontColor_G );

	QLabel* label_B = new QLabel( codec->toUnicode( "B:" ), this );
	ColorLayout->addWidget( label_B );

    FontColor_B = new QSpinBox( this, "FontColor_B" );
    FontColor_B->setMaxValue(255);
    FontColor_B->setMinValue(0);
	ColorLayout->addWidget( FontColor_B );

    FontSetting->addLayout( ColorLayout, 2, 1 );

	OptionLayout->addLayout( FontSetting );

    DayFrame = new QFrame( this, "DayFrame" );
    DayFrame->setFrameStyle( QFrame::HLine | QFrame::Sunken );
    OptionLayout->addWidget( DayFrame );

    DaySetting = new QGridLayout; 
    DaySetting->setSpacing( 3 );
    DaySetting->setMargin( 0 );

	QLabel* label2_1 = new QLabel( codec->toUnicode( "ǯɽ" ), this );
	DaySetting->addWidget( label2_1, 0, 0 );
    DayEnable = new QComboBox( false, this );
	DaySetting->addWidget( DayEnable, 0, 1 );
	for( i= 0 ; i < 2 ; i++ )
		DayEnable->insertItem( codec->toUnicode( item1[i] ), i );

	QLabel* label2_2 = new QLabel( codec->toUnicode( "  ǯɽ" ), this );
	DaySetting->addWidget( label2_2, 1, 0 );
    YearView = new QComboBox( false, this );
	DaySetting->addWidget( YearView, 1, 1 );
	for( i= 0 ; i < 2 ; i++ )
		YearView->insertItem( codec->toUnicode( item1[i] ), i );

	QLabel* label2_3 = new QLabel( codec->toUnicode( "    ǯɽ" ), this );
	DaySetting->addWidget( label2_3, 2, 0 );
    YearFull = new QComboBox( false, this );
	DaySetting->addWidget( YearFull, 2, 1 );
	for( i= 0 ; i < 2 ; i++ )
		YearFull->insertItem( codec->toUnicode( item2[i] ), i );

	QLabel* label2_8 = new QLabel( codec->toUnicode( "    " ), this );
	DaySetting->addWidget( label2_8, 3, 0 );
    YearEto = new QComboBox( false, this );
	DaySetting->addWidget( YearEto, 3, 1 );
	for( i= 0 ; i < 2 ; i++ )
		YearEto->insertItem( codec->toUnicode( item1[i] ), i );

	QLabel* label2_4 = new QLabel( codec->toUnicode( " ɽ" ), this );
	DaySetting->addWidget( label2_4, 1, 2 );
    WeekView = new QComboBox( false, this );
	DaySetting->addWidget( WeekView, 1, 3 );
	for( i= 0 ; i < 2 ; i++ )
		WeekView->insertItem( codec->toUnicode( item1[i] ), i );

	QLabel* label2_5 = new QLabel( codec->toUnicode( "   ɽ" ), this );
	DaySetting->addWidget( label2_5, 2, 2 );
    WeekJapan = new QComboBox( false, this );
	DaySetting->addWidget( WeekJapan, 2, 3 );
	for( i= 0 ; i < 2 ; i++ )
		WeekJapan->insertItem( codec->toUnicode( item3[i] ), i );

	QLabel* label2_7 = new QLabel( codec->toUnicode( "ɽ" ), this );
	DaySetting->addWidget( label2_7, 4, 0 );
    SecView = new QComboBox( false, this );
	DaySetting->addWidget( SecView, 4, 1 );
	for( i= 0 ; i < 2 ; i++ )
		SecView->insertItem( codec->toUnicode( item1[i] ), i );

	QLabel* label2_6 = new QLabel( codec->toUnicode( " " ), this );
	DaySetting->addWidget( label2_6, 4, 2 );
    WordWrap = new QComboBox( false, this );
	DaySetting->addWidget( WordWrap, 4, 3 );
	for( i= 0 ; i < 2 ; i++ )
		WordWrap->insertItem( codec->toUnicode( item1[i] ), i );

	OptionLayout->addLayout( DaySetting );

	QFontDatabase	*font_db = new QFontDatabase();
	FontName->insertStringList( font_db->families() );
	updateSizes();
	currentFont = parent->font();

    connect( FontName, SIGNAL( activated(const QString&) ), this, SLOT( updateSizes() ) );

}

void OptionDialog::accept()
{
	QDialog::accept();
}

void OptionDialog::setFontName( QString family_name )
{
	for( int i = 0 ; i < FontName->count() ; i++ )
	{
		// check same name font
		if ( ( FontName->text( i ) ) == family_name )
		{
			FontName->setCurrentItem( i );
			break;
		}
	}
	updateSizes();

}

void OptionDialog::setFontSize( int num )
{
	for( int i = 0 ; i < FontSize->count() ; i++ )
	{
		// check same name font
		if ( ( FontSize->text( i ) ) == num )
		{
			FontSize->setCurrentItem( i );
			break;
		}
	}
}

void OptionDialog::setFontColor_R( int num )
{
	FontColor_R->setValue( num );
}

void OptionDialog::setFontColor_G( int num )
{
	FontColor_G->setValue( num );
}

void OptionDialog::setFontColor_B( int num )
{
	FontColor_B->setValue( num );
}

void OptionDialog::setDayEnable( bool flg )
{
	DayEnable->setCurrentItem( flg );
}

void OptionDialog::setYearView( bool flg )
{
	YearView->setCurrentItem( flg );
}

void OptionDialog::setYearFull( bool flg )
{
	YearFull->setCurrentItem( flg );
}

void OptionDialog::setWeekView( bool flg )
{
	WeekView->setCurrentItem( flg );
}

void OptionDialog::setWeekJapan( bool flg )
{
	WeekJapan->setCurrentItem( flg );
}

void OptionDialog::setWordWrap( bool flg )
{
	WordWrap->setCurrentItem( flg );
}

void OptionDialog::setSecView( bool flg )
{
	SecView->setCurrentItem( flg );
}

void OptionDialog::setYearEto( bool flg )
{
	YearEto->setCurrentItem( flg );
}

QString  OptionDialog::getFontName()
{
	return( FontName->currentText() );
}

int  OptionDialog::getFontSize()
{
	int num = FontSize->currentText().toInt();
	
	return( num );
}

int  OptionDialog::getFontColor_R()
{
	int num = FontColor_R->value();
	return( num );
}

int  OptionDialog::getFontColor_G()
{
	int num = FontColor_G->value();
	return( num );
}

int  OptionDialog::getFontColor_B()
{
	int num = FontColor_B->value();
	return( num );
}

bool  OptionDialog::getDayEnable()
{
	return( DayEnable->currentItem() );
}

bool  OptionDialog::getYearView()
{
	return( YearView->currentItem() );
}

bool  OptionDialog::getYearFull()
{
	return( YearFull->currentItem() );
}

bool  OptionDialog::getWeekView()
{
	return( WeekView->currentItem() );
}

bool  OptionDialog::getWeekJapan()
{
	return( WeekJapan->currentItem() );
}

bool  OptionDialog::getWordWrap()
{
	return( WordWrap->currentItem() );
}

bool  OptionDialog::getSecView()
{
	return( SecView->currentItem() );
}

bool  OptionDialog::getYearEto()
{
	return( YearEto->currentItem() );
}

void OptionDialog::updateSizes()
{
	QFontDatabase fontDB;
	int currentSize = -1;
    int f = 10;

	if (FontName->currentText().lower() == currentFont.family().lower())
		currentSize = currentFont.pointSize();

	FontSize->clear();
	QValueList<int> sizes = fontDB.pointSizes(FontName->currentText());
	for(QValueList<int>::Iterator it = sizes.begin(); it != sizes.end(); it++)
	{
		FontSize->insertItem(QString::number(*it/f));
		if (currentSize != -1 && (int)((*it)/f) == currentSize)
			FontSize->setCurrentItem(FontSize->count() - 1);
	}
}


OptionDialog::~OptionDialog()
{
    // no need to delete child widgets, Qt does it all for us
}
